# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, List, Tuple, cast

import pytest

from giskard_hub import HubClient, AsyncHubClient
from tests.utils import assert_matches_type
from giskard_hub.types import APIPaginatedMetadata
from giskard_hub.types.evaluation import TestCaseEvaluation

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestResults:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: HubClient) -> None:
        result = client.evaluations.results.retrieve(
            result_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            evaluation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(TestCaseEvaluation, result, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve_with_all_params(self, client: HubClient) -> None:
        result = client.evaluations.results.retrieve(
            result_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            evaluation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            include=["test_case"],
        )
        assert_matches_type(TestCaseEvaluation, result, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: HubClient) -> None:
        response = client.evaluations.results.with_raw_response.retrieve(
            result_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            evaluation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        result = response.parse()
        assert_matches_type(TestCaseEvaluation, result, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: HubClient) -> None:
        with client.evaluations.results.with_streaming_response.retrieve(
            result_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            evaluation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            result = response.parse()
            assert_matches_type(TestCaseEvaluation, result, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: HubClient) -> None:
        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `evaluation_id` but received ''",
        ):
            client.evaluations.results.with_raw_response.retrieve(
                result_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                evaluation_id="",
            )

        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `result_id` but received ''",
        ):
            client.evaluations.results.with_raw_response.retrieve(
                result_id="",
                evaluation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update(self, client: HubClient) -> None:
        result = client.evaluations.results.update(
            result_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            evaluation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(TestCaseEvaluation, result, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: HubClient) -> None:
        result = client.evaluations.results.update(
            result_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            evaluation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            failure_category={
                "description": "description",
                "identifier": "identifier",
                "title": "title",
            },
        )
        assert_matches_type(TestCaseEvaluation, result, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: HubClient) -> None:
        response = client.evaluations.results.with_raw_response.update(
            result_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            evaluation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        result = response.parse()
        assert_matches_type(TestCaseEvaluation, result, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: HubClient) -> None:
        with client.evaluations.results.with_streaming_response.update(
            result_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            evaluation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            result = response.parse()
            assert_matches_type(TestCaseEvaluation, result, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_update(self, client: HubClient) -> None:
        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `evaluation_id` but received ''",
        ):
            client.evaluations.results.with_raw_response.update(
                result_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                evaluation_id="",
            )

        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `result_id` but received ''",
        ):
            client.evaluations.results.with_raw_response.update(
                result_id="",
                evaluation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: HubClient) -> None:
        result = client.evaluations.results.list(
            evaluation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(List[TestCaseEvaluation], result, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: HubClient) -> None:
        result = client.evaluations.results.list(
            evaluation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            include=["test_case"],
        )
        assert_matches_type(List[TestCaseEvaluation], result, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: HubClient) -> None:
        response = client.evaluations.results.with_raw_response.list(
            evaluation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        result = response.parse()
        assert_matches_type(List[TestCaseEvaluation], result, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: HubClient) -> None:
        with client.evaluations.results.with_streaming_response.list(
            evaluation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            result = response.parse()
            assert_matches_type(List[TestCaseEvaluation], result, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_list(self, client: HubClient) -> None:
        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `evaluation_id` but received ''",
        ):
            client.evaluations.results.with_raw_response.list(
                evaluation_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_rerun_test_case(self, client: HubClient) -> None:
        result = client.evaluations.results.rerun_test_case(
            result_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            evaluation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(TestCaseEvaluation, result, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_rerun_test_case(self, client: HubClient) -> None:
        response = client.evaluations.results.with_raw_response.rerun_test_case(
            result_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            evaluation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        result = response.parse()
        assert_matches_type(TestCaseEvaluation, result, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_rerun_test_case(self, client: HubClient) -> None:
        with client.evaluations.results.with_streaming_response.rerun_test_case(
            result_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            evaluation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            result = response.parse()
            assert_matches_type(TestCaseEvaluation, result, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_rerun_test_case(self, client: HubClient) -> None:
        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `evaluation_id` but received ''",
        ):
            client.evaluations.results.with_raw_response.rerun_test_case(
                result_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                evaluation_id="",
            )

        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `result_id` but received ''",
        ):
            client.evaluations.results.with_raw_response.rerun_test_case(
                result_id="",
                evaluation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_submit_local_output(self, client: HubClient) -> None:
        result = client.evaluations.results.submit_local_output(
            result_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            evaluation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(TestCaseEvaluation, result, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_submit_local_output_with_all_params(self, client: HubClient) -> None:
        result = client.evaluations.results.submit_local_output(
            result_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            evaluation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            error="error",
            agent_output={
                "response": {
                    "content": "content",
                    "role": "role",
                },
                "error": {
                    "message": "message",
                    "details": {"foo": "bar"},
                },
                "metadata": {"foo": "bar"},
            },
        )
        assert_matches_type(TestCaseEvaluation, result, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_submit_local_output(self, client: HubClient) -> None:
        response = client.evaluations.results.with_raw_response.submit_local_output(
            result_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            evaluation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        result = response.parse()
        assert_matches_type(TestCaseEvaluation, result, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_submit_local_output(self, client: HubClient) -> None:
        with client.evaluations.results.with_streaming_response.submit_local_output(
            result_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            evaluation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            result = response.parse()
            assert_matches_type(TestCaseEvaluation, result, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_submit_local_output(self, client: HubClient) -> None:
        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `evaluation_id` but received ''",
        ):
            client.evaluations.results.with_raw_response.submit_local_output(
                result_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                evaluation_id="",
            )

        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `result_id` but received ''",
        ):
            client.evaluations.results.with_raw_response.submit_local_output(
                result_id="",
                evaluation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_search(self, client: HubClient) -> None:
        result = client.evaluations.results.search(
            evaluation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(List[TestCaseEvaluation], result, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_search_with_all_params(self, client: HubClient) -> None:
        result = client.evaluations.results.search(
            evaluation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            query="query",
            filters={},
            limit=20,
            offset=0,
            include=["test_case"],
        )
        assert_matches_type(List[TestCaseEvaluation], result, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_search_with_metadata(self, client: HubClient) -> None:
        result = client.evaluations.results.search(
            evaluation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            include_metadata=True,
        )
        assert_matches_type(Tuple[List[TestCaseEvaluation], APIPaginatedMetadata], result, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_search(self, client: HubClient) -> None:
        response = client.evaluations.results.with_raw_response.search(
            evaluation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        result = response.parse()
        assert_matches_type(List[TestCaseEvaluation], result, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_search(self, client: HubClient) -> None:
        with client.evaluations.results.with_streaming_response.search(
            evaluation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            result = response.parse()
            assert_matches_type(List[TestCaseEvaluation], result, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_search(self, client: HubClient) -> None:
        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `evaluation_id` but received ''",
        ):
            client.evaluations.results.with_raw_response.search(
                evaluation_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update_visibility(self, client: HubClient) -> None:
        result = client.evaluations.results.update_visibility(
            result_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            evaluation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            hidden=False,
        )
        assert_matches_type(TestCaseEvaluation, result, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_update_visibility(self, client: HubClient) -> None:
        response = client.evaluations.results.with_raw_response.update_visibility(
            result_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            evaluation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            hidden=False,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        result = response.parse()
        assert_matches_type(TestCaseEvaluation, result, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_update_visibility(self, client: HubClient) -> None:
        with client.evaluations.results.with_streaming_response.update_visibility(
            result_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            evaluation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            hidden=False,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            result = response.parse()
            assert_matches_type(TestCaseEvaluation, result, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_update_visibility(self, client: HubClient) -> None:
        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `evaluation_id` but received ''",
        ):
            client.evaluations.results.with_raw_response.update_visibility(
                result_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                evaluation_id="",
                hidden=False,
            )

        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `result_id` but received ''",
        ):
            client.evaluations.results.with_raw_response.update_visibility(
                result_id="",
                evaluation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                hidden=False,
            )


class TestAsyncResults:
    parametrize = pytest.mark.parametrize(
        "async_client",
        [False, True, {"http_client": "aiohttp"}],
        indirect=True,
        ids=["loose", "strict", "aiohttp"],
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncHubClient) -> None:
        result = await async_client.evaluations.results.retrieve(
            result_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            evaluation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(TestCaseEvaluation, result, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve_with_all_params(self, async_client: AsyncHubClient) -> None:
        result = await async_client.evaluations.results.retrieve(
            result_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            evaluation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            include=["test_case"],
        )
        assert_matches_type(TestCaseEvaluation, result, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncHubClient) -> None:
        response = await async_client.evaluations.results.with_raw_response.retrieve(
            result_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            evaluation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        result = await response.parse()
        assert_matches_type(TestCaseEvaluation, result, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncHubClient) -> None:
        async with async_client.evaluations.results.with_streaming_response.retrieve(
            result_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            evaluation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            result = await response.parse()
            assert_matches_type(TestCaseEvaluation, result, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncHubClient) -> None:
        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `evaluation_id` but received ''",
        ):
            await async_client.evaluations.results.with_raw_response.retrieve(
                result_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                evaluation_id="",
            )

        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `result_id` but received ''",
        ):
            await async_client.evaluations.results.with_raw_response.retrieve(
                result_id="",
                evaluation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncHubClient) -> None:
        result = await async_client.evaluations.results.update(
            result_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            evaluation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(TestCaseEvaluation, result, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncHubClient) -> None:
        result = await async_client.evaluations.results.update(
            result_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            evaluation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            failure_category={
                "description": "description",
                "identifier": "identifier",
                "title": "title",
            },
        )
        assert_matches_type(TestCaseEvaluation, result, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncHubClient) -> None:
        response = await async_client.evaluations.results.with_raw_response.update(
            result_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            evaluation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        result = await response.parse()
        assert_matches_type(TestCaseEvaluation, result, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncHubClient) -> None:
        async with async_client.evaluations.results.with_streaming_response.update(
            result_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            evaluation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            result = await response.parse()
            assert_matches_type(TestCaseEvaluation, result, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncHubClient) -> None:
        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `evaluation_id` but received ''",
        ):
            await async_client.evaluations.results.with_raw_response.update(
                result_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                evaluation_id="",
            )

        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `result_id` but received ''",
        ):
            await async_client.evaluations.results.with_raw_response.update(
                result_id="",
                evaluation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncHubClient) -> None:
        result = await async_client.evaluations.results.list(
            evaluation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(List[TestCaseEvaluation], result, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncHubClient) -> None:
        result = await async_client.evaluations.results.list(
            evaluation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            include=["test_case"],
        )
        assert_matches_type(List[TestCaseEvaluation], result, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncHubClient) -> None:
        response = await async_client.evaluations.results.with_raw_response.list(
            evaluation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        result = await response.parse()
        assert_matches_type(List[TestCaseEvaluation], result, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncHubClient) -> None:
        async with async_client.evaluations.results.with_streaming_response.list(
            evaluation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            result = await response.parse()
            assert_matches_type(List[TestCaseEvaluation], result, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_list(self, async_client: AsyncHubClient) -> None:
        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `evaluation_id` but received ''",
        ):
            await async_client.evaluations.results.with_raw_response.list(
                evaluation_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_rerun_test_case(self, async_client: AsyncHubClient) -> None:
        result = await async_client.evaluations.results.rerun_test_case(
            result_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            evaluation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(TestCaseEvaluation, result, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_rerun_test_case(self, async_client: AsyncHubClient) -> None:
        response = await async_client.evaluations.results.with_raw_response.rerun_test_case(
            result_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            evaluation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        result = await response.parse()
        assert_matches_type(TestCaseEvaluation, result, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_rerun_test_case(self, async_client: AsyncHubClient) -> None:
        async with async_client.evaluations.results.with_streaming_response.rerun_test_case(
            result_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            evaluation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            result = await response.parse()
            assert_matches_type(TestCaseEvaluation, result, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_rerun_test_case(self, async_client: AsyncHubClient) -> None:
        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `evaluation_id` but received ''",
        ):
            await async_client.evaluations.results.with_raw_response.rerun_test_case(
                result_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                evaluation_id="",
            )

        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `result_id` but received ''",
        ):
            await async_client.evaluations.results.with_raw_response.rerun_test_case(
                result_id="",
                evaluation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_submit_local_output(self, async_client: AsyncHubClient) -> None:
        result = await async_client.evaluations.results.submit_local_output(
            result_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            evaluation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(TestCaseEvaluation, result, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_submit_local_output_with_all_params(self, async_client: AsyncHubClient) -> None:
        result = await async_client.evaluations.results.submit_local_output(
            result_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            evaluation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            error="error",
            agent_output={
                "response": {
                    "content": "content",
                    "role": "role",
                },
                "error": {
                    "message": "message",
                    "details": {"foo": "bar"},
                },
                "metadata": {"foo": "bar"},
            },
        )
        assert_matches_type(TestCaseEvaluation, result, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_submit_local_output(self, async_client: AsyncHubClient) -> None:
        response = await async_client.evaluations.results.with_raw_response.submit_local_output(
            result_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            evaluation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        result = await response.parse()
        assert_matches_type(TestCaseEvaluation, result, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_submit_local_output(self, async_client: AsyncHubClient) -> None:
        async with async_client.evaluations.results.with_streaming_response.submit_local_output(
            result_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            evaluation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            result = await response.parse()
            assert_matches_type(TestCaseEvaluation, result, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_submit_local_output(self, async_client: AsyncHubClient) -> None:
        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `evaluation_id` but received ''",
        ):
            await async_client.evaluations.results.with_raw_response.submit_local_output(
                result_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                evaluation_id="",
            )

        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `result_id` but received ''",
        ):
            await async_client.evaluations.results.with_raw_response.submit_local_output(
                result_id="",
                evaluation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_search(self, async_client: AsyncHubClient) -> None:
        result = await async_client.evaluations.results.search(
            evaluation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(List[TestCaseEvaluation], result, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_search_with_all_params(self, async_client: AsyncHubClient) -> None:
        result = await async_client.evaluations.results.search(
            evaluation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            query="query",
            filters={},
            limit=20,
            offset=0,
            include=["test_case"],
        )
        assert_matches_type(List[TestCaseEvaluation], result, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_search_with_metadata(self, async_client: AsyncHubClient) -> None:
        result = await async_client.evaluations.results.search(
            evaluation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            include_metadata=True,
        )
        assert_matches_type(Tuple[List[TestCaseEvaluation], APIPaginatedMetadata], result, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_search(self, async_client: AsyncHubClient) -> None:
        response = await async_client.evaluations.results.with_raw_response.search(
            evaluation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        result = await response.parse()
        assert_matches_type(List[TestCaseEvaluation], result, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_search(self, async_client: AsyncHubClient) -> None:
        async with async_client.evaluations.results.with_streaming_response.search(
            evaluation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            result = await response.parse()
            assert_matches_type(List[TestCaseEvaluation], result, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_search(self, async_client: AsyncHubClient) -> None:
        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `evaluation_id` but received ''",
        ):
            await async_client.evaluations.results.with_raw_response.search(
                evaluation_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update_visibility(self, async_client: AsyncHubClient) -> None:
        result = await async_client.evaluations.results.update_visibility(
            result_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            evaluation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            hidden=False,
        )
        assert_matches_type(TestCaseEvaluation, result, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_update_visibility(self, async_client: AsyncHubClient) -> None:
        response = await async_client.evaluations.results.with_raw_response.update_visibility(
            result_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            evaluation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            hidden=False,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        result = await response.parse()
        assert_matches_type(TestCaseEvaluation, result, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_update_visibility(self, async_client: AsyncHubClient) -> None:
        async with async_client.evaluations.results.with_streaming_response.update_visibility(
            result_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            evaluation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            hidden=False,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            result = await response.parse()
            assert_matches_type(TestCaseEvaluation, result, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_update_visibility(self, async_client: AsyncHubClient) -> None:
        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `evaluation_id` but received ''",
        ):
            await async_client.evaluations.results.with_raw_response.update_visibility(
                result_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                evaluation_id="",
                hidden=False,
            )

        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `result_id` but received ''",
        ):
            await async_client.evaluations.results.with_raw_response.update_visibility(
                result_id="",
                evaluation_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                hidden=False,
            )



from __future__ import annotations

from typing import Any, Dict, List, TypeVar, Callable, Iterable, cast

from ..types.common import APIResponse, APIPaginatedResponse, APIResponseWithIncluded

T = TypeVar("T")


def _unwrap(value: Any) -> Any:
    """Return ``value.data`` if it's an APIResponse, otherwise ``value`` as-is."""
    if isinstance(value, APIResponse):
        return cast(APIResponse[Any], value).data
    return value


def _apply_included_to_item(item: object, included_for_item: Dict[str, Any]) -> None:
    """Replace reference fields on *item* with fully-resolved included resources."""
    for field_name, related in included_for_item.items():
        if not hasattr(item, field_name):
            continue

        if isinstance(related, list):
            resolved: Any = [_unwrap(el) for el in cast(List[Any], related)]
        else:
            resolved = _unwrap(related)
        setattr(item, field_name, resolved)


def _embed_into_items(
    items: Iterable[T],
    included: Dict[str, Dict[str, Any]],
    id_getter: Callable[[T], str],
) -> None:
    """Shared loop: embed included resources into each item in *items*."""
    for item in items:
        included_for_item = included.get(id_getter(item))
        if included_for_item:
            _apply_included_to_item(item, included_for_item)


def embed_included_single(
    response: APIResponseWithIncluded[T, Any],
    *,
    id_getter: Callable[[T], str],
) -> APIResponseWithIncluded[T, Any]:
    """Embed included resources into a single primary object (mutates *response*)."""
    if response.included:
        included_for_item = response.included.get(id_getter(response.data))
        if included_for_item:
            _apply_included_to_item(response.data, included_for_item)
    return response


def embed_included_list(
    response: APIResponseWithIncluded[List[T], Any],
    *,
    id_getter: Callable[[T], str],
) -> APIResponseWithIncluded[List[T], Any]:
    """Embed included resources into each primary object in a list (mutates *response*)."""
    if response.included:
        _embed_into_items(response.data, response.included, id_getter)
    return response


def embed_included_paginated(
    response: APIPaginatedResponse[T, Any],
    *,
    id_getter: Callable[[T], str],
) -> APIPaginatedResponse[T, Any]:
    """Embed included resources into each primary object in a paginated response (mutates *response*)."""
    if response.included:
        _embed_into_items(response.data, response.included, id_getter)
    return response



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, List, Tuple, cast

import pytest

from giskard_hub import HubClient, AsyncHubClient
from tests.utils import assert_matches_type
from giskard_hub.types import (
    Dataset,
    TestCase,
    APIPaginatedMetadata,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestDatasets:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: HubClient) -> None:
        dataset = client.datasets.create(
            name="name",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(Dataset, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: HubClient) -> None:
        dataset = client.datasets.create(
            name="name",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            description="description",
        )
        assert_matches_type(Dataset, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: HubClient) -> None:
        response = client.datasets.with_raw_response.create(
            name="name",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        dataset = response.parse()
        assert_matches_type(Dataset, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: HubClient) -> None:
        with client.datasets.with_streaming_response.create(
            name="name",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            dataset = response.parse()
            assert_matches_type(Dataset, dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_upload(self, client: HubClient) -> None:
        dataset = client.datasets.upload(
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            data=[{"id": "1", "value": "foo"}],
        )
        assert_matches_type(Dataset, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_upload_with_all_params(self, client: HubClient) -> None:
        dataset = client.datasets.upload(
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            data=[{"id": "1", "value": "foo"}],
            dataset_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            name="my-dataset",
        )
        assert_matches_type(Dataset, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_upload(self, client: HubClient) -> None:
        response = client.datasets.with_raw_response.upload(
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            data=[{"id": "1", "value": "foo"}],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        dataset = response.parse()
        assert_matches_type(Dataset, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_upload(self, client: HubClient) -> None:
        with client.datasets.with_streaming_response.upload(
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            data=[{"id": "1", "value": "foo"}],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            dataset = response.parse()
            assert_matches_type(Dataset, dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: HubClient) -> None:
        dataset = client.datasets.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(Dataset, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: HubClient) -> None:
        response = client.datasets.with_raw_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        dataset = response.parse()
        assert_matches_type(Dataset, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: HubClient) -> None:
        with client.datasets.with_streaming_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            dataset = response.parse()
            assert_matches_type(Dataset, dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: HubClient) -> None:
        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `dataset_id` but received ''",
        ):
            client.datasets.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update(self, client: HubClient) -> None:
        dataset = client.datasets.update(
            dataset_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(Dataset, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: HubClient) -> None:
        dataset = client.datasets.update(
            dataset_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            description="description",
            name="name",
            status={
                "total": 0,
                "current": 0,
                "error": "error",
                "state": "skipped",
            },
        )
        assert_matches_type(Dataset, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: HubClient) -> None:
        response = client.datasets.with_raw_response.update(
            dataset_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        dataset = response.parse()
        assert_matches_type(Dataset, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: HubClient) -> None:
        with client.datasets.with_streaming_response.update(
            dataset_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            dataset = response.parse()
            assert_matches_type(Dataset, dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_update(self, client: HubClient) -> None:
        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `dataset_id` but received ''",
        ):
            client.datasets.with_raw_response.update(
                dataset_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: HubClient) -> None:
        dataset = client.datasets.list()
        assert_matches_type(List[Dataset], dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: HubClient) -> None:
        dataset = client.datasets.list(
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(List[Dataset], dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: HubClient) -> None:
        response = client.datasets.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        dataset = response.parse()
        assert_matches_type(List[Dataset], dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: HubClient) -> None:
        with client.datasets.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            dataset = response.parse()
            assert_matches_type(List[Dataset], dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_delete(self, client: HubClient) -> None:
        dataset = client.datasets.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(type(None), dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: HubClient) -> None:
        response = client.datasets.with_raw_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        dataset = response.parse()
        assert_matches_type(type(None), dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: HubClient) -> None:
        with client.datasets.with_streaming_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            dataset = response.parse()
            assert_matches_type(type(None), dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: HubClient) -> None:
        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `dataset_id` but received ''",
        ):
            client.datasets.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_bulk_delete(self, client: HubClient) -> None:
        dataset = client.datasets.bulk_delete(
            dataset_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
        )
        assert_matches_type(type(None), dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_bulk_delete(self, client: HubClient) -> None:
        response = client.datasets.with_raw_response.bulk_delete(
            dataset_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        dataset = response.parse()
        assert_matches_type(type(None), dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_bulk_delete(self, client: HubClient) -> None:
        with client.datasets.with_streaming_response.bulk_delete(
            dataset_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            dataset = response.parse()
            assert_matches_type(type(None), dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_generate_scenario_based(self, client: HubClient) -> None:
        dataset = client.datasets.generate_scenario_based(
            agent_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            scenario_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(Dataset, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_generate_scenario_based_with_all_params(self, client: HubClient) -> None:
        dataset = client.datasets.generate_scenario_based(
            agent_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            scenario_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            n_examples=10,
            dataset_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            dataset_name="dataset_name",
        )
        assert_matches_type(Dataset, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_generate_scenario_based(self, client: HubClient) -> None:
        response = client.datasets.with_raw_response.generate_scenario_based(
            agent_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            scenario_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        dataset = response.parse()
        assert_matches_type(Dataset, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_generate_scenario_based(self, client: HubClient) -> None:
        with client.datasets.with_streaming_response.generate_scenario_based(
            agent_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            scenario_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            dataset = response.parse()
            assert_matches_type(Dataset, dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_generate_document_based(self, client: HubClient) -> None:
        dataset = client.datasets.generate_document_based(
            agent_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            knowledge_base_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(Dataset, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_generate_document_based_with_all_params(self, client: HubClient) -> None:
        dataset = client.datasets.generate_document_based(
            agent_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            knowledge_base_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            dataset_name="dataset_name",
            description="description",
            n_examples=0,
            topic_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
        )
        assert_matches_type(Dataset, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_generate_document_based(self, client: HubClient) -> None:
        response = client.datasets.with_raw_response.generate_document_based(
            agent_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            knowledge_base_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        dataset = response.parse()
        assert_matches_type(Dataset, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_generate_document_based(self, client: HubClient) -> None:
        with client.datasets.with_streaming_response.generate_document_based(
            agent_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            knowledge_base_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            dataset = response.parse()
            assert_matches_type(Dataset, dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_tags(self, client: HubClient) -> None:
        dataset = client.datasets.list_tags(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(List[str], dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list_tags(self, client: HubClient) -> None:
        response = client.datasets.with_raw_response.list_tags(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        dataset = response.parse()
        assert_matches_type(List[str], dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list_tags(self, client: HubClient) -> None:
        with client.datasets.with_streaming_response.list_tags(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            dataset = response.parse()
            assert_matches_type(List[str], dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_list_tags(self, client: HubClient) -> None:
        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `dataset_id` but received ''",
        ):
            client.datasets.with_raw_response.list_tags(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_test_cases(self, client: HubClient) -> None:
        dataset = client.datasets.list_test_cases(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(List[TestCase], dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list_test_cases(self, client: HubClient) -> None:
        response = client.datasets.with_raw_response.list_test_cases(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        dataset = response.parse()
        assert_matches_type(List[TestCase], dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list_test_cases(self, client: HubClient) -> None:
        with client.datasets.with_streaming_response.list_test_cases(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            dataset = response.parse()
            assert_matches_type(List[TestCase], dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_list_test_cases(self, client: HubClient) -> None:
        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `dataset_id` but received ''",
        ):
            client.datasets.with_raw_response.list_test_cases(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_search_test_cases(self, client: HubClient) -> None:
        result = client.datasets.search_test_cases(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(List[TestCase], result, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_search_test_cases_with_all_params(self, client: HubClient) -> None:
        result = client.datasets.search_test_cases(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            filters={
                "status": {
                    "selected_options": ["active", "draft"],
                    "match_logic": "any",
                },
                "tags": {
                    "selected_options": ["tag1", "tag2"],
                    "match_logic": "any",
                },
            },
            order_by=[
                {
                    "id": "created_at",
                    "desc": True,
                }
            ],
            limit=10,
            offset=0,
        )
        assert_matches_type(List[TestCase], result, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_search_test_cases_with_metadata(self, client: HubClient) -> None:
        result = client.datasets.search_test_cases(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            include_metadata=True,
        )
        assert_matches_type(Tuple[List[TestCase], APIPaginatedMetadata], result, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_search_test_cases(self, client: HubClient) -> None:
        response = client.datasets.with_raw_response.search_test_cases(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        result = response.parse()
        assert_matches_type(List[TestCase], result, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_search_test_cases(self, client: HubClient) -> None:
        with client.datasets.with_streaming_response.search_test_cases(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            result = response.parse()
            assert_matches_type(List[TestCase], result, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_search_test_cases(self, client: HubClient) -> None:
        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `dataset_id` but received ''",
        ):
            client.datasets.with_raw_response.search_test_cases(
                "",
            )


class TestAsyncDatasets:
    parametrize = pytest.mark.parametrize(
        "async_client",
        [False, True, {"http_client": "aiohttp"}],
        indirect=True,
        ids=["loose", "strict", "aiohttp"],
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncHubClient) -> None:
        dataset = await async_client.datasets.create(
            name="name",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(Dataset, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncHubClient) -> None:
        dataset = await async_client.datasets.create(
            name="name",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            description="description",
        )
        assert_matches_type(Dataset, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncHubClient) -> None:
        response = await async_client.datasets.with_raw_response.create(
            name="name",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        dataset = await response.parse()
        assert_matches_type(Dataset, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncHubClient) -> None:
        async with async_client.datasets.with_streaming_response.create(
            name="name",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            dataset = await response.parse()
            assert_matches_type(Dataset, dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_upload(self, async_client: AsyncHubClient) -> None:
        dataset = await async_client.datasets.upload(
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            data=b"raw file contents",
        )
        assert_matches_type(Dataset, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_upload_with_all_params(self, async_client: AsyncHubClient) -> None:
        dataset = await async_client.datasets.upload(
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            data=b"raw file contents",
            dataset_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            name="my-dataset",
        )
        assert_matches_type(Dataset, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_upload(self, async_client: AsyncHubClient) -> None:
        response = await async_client.datasets.with_raw_response.upload(
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            data=b"raw file contents",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        dataset = await response.parse()
        assert_matches_type(Dataset, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_upload(self, async_client: AsyncHubClient) -> None:
        async with async_client.datasets.with_streaming_response.upload(
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            data=b"raw file contents",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            dataset = await response.parse()
            assert_matches_type(Dataset, dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncHubClient) -> None:
        dataset = await async_client.datasets.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(Dataset, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncHubClient) -> None:
        response = await async_client.datasets.with_raw_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        dataset = await response.parse()
        assert_matches_type(Dataset, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncHubClient) -> None:
        async with async_client.datasets.with_streaming_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            dataset = await response.parse()
            assert_matches_type(Dataset, dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncHubClient) -> None:
        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `dataset_id` but received ''",
        ):
            await async_client.datasets.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncHubClient) -> None:
        dataset = await async_client.datasets.update(
            dataset_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(Dataset, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncHubClient) -> None:
        dataset = await async_client.datasets.update(
            dataset_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            description="description",
            name="name",
            status={
                "total": 0,
                "current": 0,
                "error": "error",
                "state": "skipped",
            },
        )
        assert_matches_type(Dataset, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncHubClient) -> None:
        response = await async_client.datasets.with_raw_response.update(
            dataset_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        dataset = await response.parse()
        assert_matches_type(Dataset, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncHubClient) -> None:
        async with async_client.datasets.with_streaming_response.update(
            dataset_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            dataset = await response.parse()
            assert_matches_type(Dataset, dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncHubClient) -> None:
        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `dataset_id` but received ''",
        ):
            await async_client.datasets.with_raw_response.update(
                dataset_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncHubClient) -> None:
        dataset = await async_client.datasets.list()
        assert_matches_type(List[Dataset], dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncHubClient) -> None:
        dataset = await async_client.datasets.list(
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(List[Dataset], dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncHubClient) -> None:
        response = await async_client.datasets.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        dataset = await response.parse()
        assert_matches_type(List[Dataset], dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncHubClient) -> None:
        async with async_client.datasets.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            dataset = await response.parse()
            assert_matches_type(List[Dataset], dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncHubClient) -> None:
        dataset = await async_client.datasets.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(type(None), dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncHubClient) -> None:
        response = await async_client.datasets.with_raw_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        dataset = await response.parse()
        assert_matches_type(type(None), dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncHubClient) -> None:
        async with async_client.datasets.with_streaming_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            dataset = await response.parse()
            assert_matches_type(type(None), dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncHubClient) -> None:
        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `dataset_id` but received ''",
        ):
            await async_client.datasets.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_bulk_delete(self, async_client: AsyncHubClient) -> None:
        dataset = await async_client.datasets.bulk_delete(
            dataset_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
        )
        assert_matches_type(type(None), dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_bulk_delete(self, async_client: AsyncHubClient) -> None:
        response = await async_client.datasets.with_raw_response.bulk_delete(
            dataset_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        dataset = await response.parse()
        assert_matches_type(type(None), dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_bulk_delete(self, async_client: AsyncHubClient) -> None:
        async with async_client.datasets.with_streaming_response.bulk_delete(
            dataset_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            dataset = await response.parse()
            assert_matches_type(type(None), dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_generate_scenario_based(self, async_client: AsyncHubClient) -> None:
        dataset = await async_client.datasets.generate_scenario_based(
            agent_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            scenario_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(Dataset, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_generate_scenario_based_with_all_params(self, async_client: AsyncHubClient) -> None:
        dataset = await async_client.datasets.generate_scenario_based(
            agent_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            scenario_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            n_examples=10,
            dataset_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            dataset_name="dataset_name",
        )
        assert_matches_type(Dataset, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_generate_scenario_based(self, async_client: AsyncHubClient) -> None:
        response = await async_client.datasets.with_raw_response.generate_scenario_based(
            agent_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            scenario_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        dataset = await response.parse()
        assert_matches_type(Dataset, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_generate_scenario_based(self, async_client: AsyncHubClient) -> None:
        async with async_client.datasets.with_streaming_response.generate_scenario_based(
            agent_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            scenario_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            dataset = await response.parse()
            assert_matches_type(Dataset, dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_generate_document_based(self, async_client: AsyncHubClient) -> None:
        dataset = await async_client.datasets.generate_document_based(
            agent_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            knowledge_base_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(Dataset, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_generate_document_based_with_all_params(self, async_client: AsyncHubClient) -> None:
        dataset = await async_client.datasets.generate_document_based(
            agent_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            knowledge_base_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            dataset_name="dataset_name",
            description="description",
            n_examples=0,
            topic_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
        )
        assert_matches_type(Dataset, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_generate_document_based(self, async_client: AsyncHubClient) -> None:
        response = await async_client.datasets.with_raw_response.generate_document_based(
            agent_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            knowledge_base_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        dataset = await response.parse()
        assert_matches_type(Dataset, dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_generate_document_based(self, async_client: AsyncHubClient) -> None:
        async with async_client.datasets.with_streaming_response.generate_document_based(
            agent_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            knowledge_base_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            dataset = await response.parse()
            assert_matches_type(Dataset, dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_tags(self, async_client: AsyncHubClient) -> None:
        dataset = await async_client.datasets.list_tags(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(List[str], dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list_tags(self, async_client: AsyncHubClient) -> None:
        response = await async_client.datasets.with_raw_response.list_tags(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        dataset = await response.parse()
        assert_matches_type(List[str], dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list_tags(self, async_client: AsyncHubClient) -> None:
        async with async_client.datasets.with_streaming_response.list_tags(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            dataset = await response.parse()
            assert_matches_type(List[str], dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_list_tags(self, async_client: AsyncHubClient) -> None:
        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `dataset_id` but received ''",
        ):
            await async_client.datasets.with_raw_response.list_tags(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_test_cases(self, async_client: AsyncHubClient) -> None:
        dataset = await async_client.datasets.list_test_cases(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(List[TestCase], dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list_test_cases(self, async_client: AsyncHubClient) -> None:
        response = await async_client.datasets.with_raw_response.list_test_cases(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        dataset = await response.parse()
        assert_matches_type(List[TestCase], dataset, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list_test_cases(self, async_client: AsyncHubClient) -> None:
        async with async_client.datasets.with_streaming_response.list_test_cases(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            dataset = await response.parse()
            assert_matches_type(List[TestCase], dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_list_test_cases(self, async_client: AsyncHubClient) -> None:
        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `dataset_id` but received ''",
        ):
            await async_client.datasets.with_raw_response.list_test_cases(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_search_test_cases(self, async_client: AsyncHubClient) -> None:
        result = await async_client.datasets.search_test_cases(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(List[TestCase], result, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_search_test_cases_with_all_params(self, async_client: AsyncHubClient) -> None:
        result = await async_client.datasets.search_test_cases(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            filters={
                "status": {
                    "selected_options": ["active", "draft"],
                    "match_logic": "any",
                },
                "tags": {
                    "selected_options": ["tag1", "tag2"],
                    "match_logic": "any",
                },
            },
            order_by=[
                {
                    "id": "created_at",
                    "desc": True,
                }
            ],
            limit=10,
            offset=0,
        )
        assert_matches_type(List[TestCase], result, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_search_test_cases_with_metadata(self, async_client: AsyncHubClient) -> None:
        result = await async_client.datasets.search_test_cases(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            include_metadata=True,
        )
        assert_matches_type(Tuple[List[TestCase], APIPaginatedMetadata], result, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_search_test_cases(self, async_client: AsyncHubClient) -> None:
        response = await async_client.datasets.with_raw_response.search_test_cases(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        result = await response.parse()
        assert_matches_type(List[TestCase], result, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_search_test_cases(self, async_client: AsyncHubClient) -> None:
        async with async_client.datasets.with_streaming_response.search_test_cases(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            result = await response.parse()
            assert_matches_type(List[TestCase], result, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_search_test_cases(self, async_client: AsyncHubClient) -> None:
        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `dataset_id` but received ''",
        ):
            await async_client.datasets.with_raw_response.search_test_cases(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, List, cast

import pytest

from giskard_hub import HubClient, AsyncHubClient
from tests.utils import assert_matches_type
from giskard_hub.types import Check

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestChecks:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: HubClient) -> None:
        check = client.checks.create(
            params={"reference": "reference"},
            description="description",
            identifier="identifier",
            name="name",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(Check, check, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: HubClient) -> None:
        response = client.checks.with_raw_response.create(
            params={"reference": "reference"},
            description="description",
            identifier="identifier",
            name="name",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        check = response.parse()
        assert_matches_type(Check, check, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: HubClient) -> None:
        with client.checks.with_streaming_response.create(
            params={"reference": "reference"},
            description="description",
            identifier="identifier",
            name="name",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            check = response.parse()
            assert_matches_type(Check, check, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: HubClient) -> None:
        check = client.checks.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(Check, check, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: HubClient) -> None:
        response = client.checks.with_raw_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        check = response.parse()
        assert_matches_type(Check, check, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: HubClient) -> None:
        with client.checks.with_streaming_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            check = response.parse()
            assert_matches_type(Check, check, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: HubClient) -> None:
        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `check_id` but received ''",
        ):
            client.checks.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update(self, client: HubClient) -> None:
        check = client.checks.update(
            check_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(Check, check, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: HubClient) -> None:
        check = client.checks.update(
            check_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            params={
                "reference": "reference",
                "type": "correctness",
            },
            description="description",
            identifier="identifier",
            name="name",
        )
        assert_matches_type(Check, check, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: HubClient) -> None:
        response = client.checks.with_raw_response.update(
            check_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        check = response.parse()
        assert_matches_type(Check, check, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: HubClient) -> None:
        with client.checks.with_streaming_response.update(
            check_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            check = response.parse()
            assert_matches_type(Check, check, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_update(self, client: HubClient) -> None:
        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `check_id` but received ''",
        ):
            client.checks.with_raw_response.update(
                check_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: HubClient) -> None:
        check = client.checks.list(
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(List[Check], check, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: HubClient) -> None:
        check = client.checks.list(
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            filter_builtin=True,
        )
        assert_matches_type(List[Check], check, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: HubClient) -> None:
        response = client.checks.with_raw_response.list(
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        check = response.parse()
        assert_matches_type(List[Check], check, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: HubClient) -> None:
        with client.checks.with_streaming_response.list(
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            check = response.parse()
            assert_matches_type(List[Check], check, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_delete(self, client: HubClient) -> None:
        check = client.checks.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(None, check, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: HubClient) -> None:
        response = client.checks.with_raw_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        check = response.parse()
        assert_matches_type(None, check, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: HubClient) -> None:
        with client.checks.with_streaming_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            check = response.parse()
            assert_matches_type(None, check, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: HubClient) -> None:
        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `check_id` but received ''",
        ):
            client.checks.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_bulk_delete(self, client: HubClient) -> None:
        check = client.checks.bulk_delete(
            check_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
        )
        assert_matches_type(None, check, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_bulk_delete(self, client: HubClient) -> None:
        response = client.checks.with_raw_response.bulk_delete(
            check_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        check = response.parse()
        assert_matches_type(None, check, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_bulk_delete(self, client: HubClient) -> None:
        with client.checks.with_streaming_response.bulk_delete(
            check_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            check = response.parse()
            assert_matches_type(None, check, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncChecks:
    parametrize = pytest.mark.parametrize(
        "async_client",
        [False, True, {"http_client": "aiohttp"}],
        indirect=True,
        ids=["loose", "strict", "aiohttp"],
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncHubClient) -> None:
        check = await async_client.checks.create(
            params={"reference": "reference"},
            description="description",
            identifier="identifier",
            name="name",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(Check, check, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncHubClient) -> None:
        response = await async_client.checks.with_raw_response.create(
            params={"reference": "reference"},
            description="description",
            identifier="identifier",
            name="name",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        check = await response.parse()
        assert_matches_type(Check, check, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncHubClient) -> None:
        async with async_client.checks.with_streaming_response.create(
            params={"reference": "reference"},
            description="description",
            identifier="identifier",
            name="name",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            check = await response.parse()
            assert_matches_type(Check, check, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncHubClient) -> None:
        check = await async_client.checks.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(Check, check, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncHubClient) -> None:
        response = await async_client.checks.with_raw_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        check = await response.parse()
        assert_matches_type(Check, check, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncHubClient) -> None:
        async with async_client.checks.with_streaming_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            check = await response.parse()
            assert_matches_type(Check, check, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncHubClient) -> None:
        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `check_id` but received ''",
        ):
            await async_client.checks.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncHubClient) -> None:
        check = await async_client.checks.update(
            check_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(Check, check, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncHubClient) -> None:
        check = await async_client.checks.update(
            check_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            params={
                "reference": "reference",
                "type": "correctness",
            },
            description="description",
            identifier="identifier",
            name="name",
        )
        assert_matches_type(Check, check, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncHubClient) -> None:
        response = await async_client.checks.with_raw_response.update(
            check_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        check = await response.parse()
        assert_matches_type(Check, check, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncHubClient) -> None:
        async with async_client.checks.with_streaming_response.update(
            check_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            check = await response.parse()
            assert_matches_type(Check, check, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncHubClient) -> None:
        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `check_id` but received ''",
        ):
            await async_client.checks.with_raw_response.update(
                check_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncHubClient) -> None:
        check = await async_client.checks.list(
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(List[Check], check, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncHubClient) -> None:
        check = await async_client.checks.list(
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            filter_builtin=True,
        )
        assert_matches_type(List[Check], check, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncHubClient) -> None:
        response = await async_client.checks.with_raw_response.list(
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        check = await response.parse()
        assert_matches_type(List[Check], check, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncHubClient) -> None:
        async with async_client.checks.with_streaming_response.list(
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            check = await response.parse()
            assert_matches_type(List[Check], check, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncHubClient) -> None:
        check = await async_client.checks.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(None, check, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncHubClient) -> None:
        response = await async_client.checks.with_raw_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        check = await response.parse()
        assert_matches_type(None, check, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncHubClient) -> None:
        async with async_client.checks.with_streaming_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            check = await response.parse()
            assert_matches_type(None, check, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncHubClient) -> None:
        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `check_id` but received ''",
        ):
            await async_client.checks.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_bulk_delete(self, async_client: AsyncHubClient) -> None:
        check = await async_client.checks.bulk_delete(
            check_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
        )
        assert_matches_type(None, check, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_bulk_delete(self, async_client: AsyncHubClient) -> None:
        response = await async_client.checks.with_raw_response.bulk_delete(
            check_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        check = await response.parse()
        assert_matches_type(None, check, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_bulk_delete(self, async_client: AsyncHubClient) -> None:
        async with async_client.checks.with_streaming_response.bulk_delete(
            check_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            check = await response.parse()
            assert_matches_type(None, check, path=["response"])

        assert cast(Any, response.is_closed) is True



"""High-level helper resources that orchestrate common workflows.

This module provides ``HelpersResource`` and ``AsyncHelpersResource``, which
wrap lower-level API resources to offer convenience methods such as
``wait_for_completion``, ``evaluate``, and ``print_metrics``.
"""

import time
import asyncio
import inspect
from typing import Callable, Optional, Awaitable, Collection, cast
from concurrent.futures import ThreadPoolExecutor

from .._types import Omit, SequenceNotStr, omit
from .._models import BaseModel
from ._display import (
    build_scan_probe_data,
    print_scan_metrics_table,
    print_evaluation_metrics_table,
)
from .._resource import SyncAPIResource, AsyncAPIResource
from ..types.chat import ChatMessage
from ..types.scan import ScanProbe, ScanProbeAttempt
from ..types.agent import Agent, AgentOutputParam
from ..types.dataset import Dataset
from ..types.project import Project
from ._helpers_types import (
    TStateful,
    AgentReturn,
    PrintMetricsEntity,
    RetrievableResource,
    AsyncRetrievableResource,
    map_entity_to_resource,
    normalize_agent_output,
)
from ..types.test_case import TestCase
from ..types.evaluation import Evaluation, TestCaseEvaluation

__all__ = ["HelpersResource", "AsyncHelpersResource"]


class HelpersResource(SyncAPIResource):
    """Synchronous high-level helpers wrapping lower-level API resources."""

    def _map_entity_to_resource(self, entity: BaseModel) -> SyncAPIResource:
        return cast(SyncAPIResource, map_entity_to_resource(self._client, entity))

    def wait_for_completion(
        self,
        entity: TStateful,
        *,
        poll_interval: float = 5.0,
        max_retries: int = 360,
        running_states: Collection[str] = frozenset({"running"}),
        error_states: Collection[str] = frozenset({"error"}),
        raise_on_error: bool = True,
    ) -> TStateful:
        """Wait until an entity leaves a running state and return its final value.

        Parameters
        ----------
        entity :
            The entity to monitor for completion.
        poll_interval :
            Time in seconds to wait between polling attempts.
        max_retries :
            Maximum number of polling attempts before timing out.
        running_states :
            Set of states considered as still running.
        error_states :
            Set of states considered as terminal error states.
        raise_on_error :
            Whether to raise an exception if an error state is reached.

        Returns
        -------
        StatefulEntity
            The refreshed entity once it has left the running states, or is in
            an error state when ``raise_on_error`` is ``False``.

        Raises
        ------
        ValueError
            If the entity reaches an error state and ``raise_on_error`` is ``True``.
        RuntimeError
            If the entity does not complete within the allotted number of retries.
        """
        resource = cast(RetrievableResource, self._map_entity_to_resource(cast(BaseModel, entity)))
        current: TStateful = entity

        for _ in range(max_retries):
            current = cast(TStateful, resource.retrieve(current.id))

            if current.state in error_states:
                if raise_on_error:
                    raise ValueError(f"Entity {current.id} reached an error state: {current.state}")
                return current

            if current.state not in running_states:
                return current

            time.sleep(poll_interval)

        raise RuntimeError(
            f"Timeout waiting for entity {current.id} to complete "
            f"after {max_retries} retries (~{max_retries * poll_interval:.0f}s)"
        )

    def evaluate(
        self,
        *,
        agent: str | Agent | Callable[[list[ChatMessage]], AgentReturn],
        dataset: str | Dataset,
        project: Optional[str | Project] | Omit = omit,
        name: Optional[str] | Omit = omit,
        tags: Optional[SequenceNotStr[str]] | Omit = omit,
    ) -> Evaluation:
        """Run an evaluation for a given agent over a dataset.

        Handles both remote agents (referenced by ID or ``Agent``) and local
        Python callables that take a list of ``ChatMessage`` and return an
        ``AgentOutput``-compatible value.

        Parameters
        ----------
        agent :
            Either a remote agent identifier (``str`` or ``Agent``) or a callable
            with signature ``(messages: list[ChatMessage]) -> AgentReturn``.
        dataset :
            Dataset identifier or ``Dataset`` instance containing the test cases
            to evaluate the agent on.
        project :
            Project identifier or ``Project`` instance.  Required when ``agent``
            is a remote agent (string or ``Agent``).
        name :
            Optional name to assign to the created evaluation.
        tags :
            Optional list of tags to filter the dataset's test cases.

        Returns
        -------
        Evaluation
            The created evaluation.

        Raises
        ------
        ValueError
            If ``project`` is not provided when running a remote evaluation.
        TypeError
            If the local agent callable returns an unsupported value, or if test
            cases do not include full ``TestCase`` objects during local evaluation.
        """
        dataset_id = dataset if isinstance(dataset, str) else dataset.id

        if isinstance(agent, (str, Agent)):
            return self._evaluate_remote(agent=agent, dataset_id=dataset_id, project=project, name=name, tags=tags)

        return self._evaluate_local(agent=agent, dataset_id=dataset_id, name=name, tags=tags)

    def print_metrics(self, entity: PrintMetricsEntity) -> None:
        """Print metrics for an evaluation or scan result to the console.

        For an evaluation, displays a table of metric names, success rates, and
        pass/fail/error/skipped counts.  For a scan result, displays probe
        categories, names, severity, and issue/attack counts.
        """
        if isinstance(entity, Evaluation):
            print_evaluation_metrics_table(entity)
        else:
            self._print_scan_metrics(entity)

    # -- Private helpers -----------------------------------------------------

    def _evaluate_remote(
        self,
        *,
        agent: str | Agent,
        dataset_id: str,
        project: Optional[str | Project] | Omit,
        name: Optional[str] | Omit,
        tags: Optional[SequenceNotStr[str]] | Omit,
    ) -> Evaluation:
        if project is omit or project is None:
            raise ValueError("Project is required when running a remote evaluation")

        project_id = project if isinstance(project, str) else cast(Project, project).id
        agent_id = agent if isinstance(agent, str) else agent.id
        name_arg: str | Omit = omit if name is None else name

        return self._client.evaluations.create(
            project_id=project_id,
            agent_id=agent_id,
            name=name_arg,
            dataset_id=dataset_id,
            tags=tags,
        )

    def _evaluate_local(
        self,
        *,
        agent: Callable[[list[ChatMessage]], AgentReturn],
        dataset_id: str,
        name: Optional[str] | Omit,
        tags: Optional[SequenceNotStr[str]] | Omit,
    ) -> Evaluation:
        evaluation = self._client.evaluations.create_local(
            agent_info={
                "name": agent.__name__,
                "description": agent.__doc__ or "",
            },
            name=name,
            dataset_id=dataset_id,
            tags=tags,
        )

        entries = self._client.evaluations.results.list(evaluation_id=evaluation.id, include=["test_case"])

        for entry in entries:
            test_case = entry.test_case
            if not isinstance(test_case, TestCase):
                raise TypeError("Expected `test_case` to be a full TestCase for local evaluation")

            agent_output_model = normalize_agent_output(agent(test_case.messages))
            agent_output_param = cast(AgentOutputParam, agent_output_model.to_dict())

            self._client.evaluations.results.submit_local_output(
                result_id=entry.id,
                evaluation_id=evaluation.id,
                agent_output=agent_output_param,
            )

        return evaluation

    def _print_scan_metrics(self, entity: object) -> None:
        from ..types.scan import Scan as _Scan

        scan = cast(_Scan, entity)
        category_map = {cat.id: cat.title for cat in self._client.scans.list_categories()}
        probe_results = self._client.scans.list_probes(scan_id=scan.id)

        def fetch_attempts(probe: ScanProbe) -> tuple[str, list[ScanProbeAttempt]]:
            return (probe.id, self._client.scans.probes.list_attempts(probe_id=probe.id))

        with ThreadPoolExecutor() as executor:
            attempts_by_probe_id = dict(executor.map(fetch_attempts, probe_results))
        probe_data = build_scan_probe_data(category_map, probe_results, attempts_by_probe_id)
        print_scan_metrics_table(probe_data, scan.id)


class AsyncHelpersResource(AsyncAPIResource):
    """Asynchronous high-level helpers wrapping lower-level API resources."""

    def _map_entity_to_resource(self, entity: BaseModel) -> AsyncAPIResource:
        return cast(AsyncAPIResource, map_entity_to_resource(self._client, entity))

    async def wait_for_completion(
        self,
        entity: TStateful,
        *,
        poll_interval: float = 5.0,
        max_retries: int = 360,
        running_states: Collection[str] = frozenset({"running"}),
        error_states: Collection[str] = frozenset({"error"}),
        raise_on_error: bool = True,
    ) -> TStateful:
        """Asynchronously wait until an entity leaves a running state.

        Parameters
        ----------
        entity :
            The entity to monitor for completion.
        poll_interval :
            Time in seconds to wait between polling attempts.
        max_retries :
            Maximum number of polling attempts before timing out.
        running_states :
            Set of states considered as still running.
        error_states :
            Set of states considered as terminal error states.
        raise_on_error :
            Whether to raise an exception if an error state is reached.

        Returns
        -------
        StatefulEntity
            The refreshed entity once it has left the running states, or is in
            an error state when ``raise_on_error`` is ``False``.

        Raises
        ------
        ValueError
            If the entity reaches an error state and ``raise_on_error`` is ``True``.
        RuntimeError
            If the entity does not complete within the allotted number of retries.
        """
        resource = cast(AsyncRetrievableResource, self._map_entity_to_resource(cast(BaseModel, entity)))
        current: TStateful = entity

        for _ in range(max_retries):
            current = cast(TStateful, await resource.retrieve(current.id))

            if current.state in error_states:
                if raise_on_error:
                    raise ValueError(f"Entity {current.id} reached an error state: {current.state}")
                return current

            if current.state not in running_states:
                return current

            await asyncio.sleep(poll_interval)

        raise RuntimeError(
            f"Timeout waiting for entity {current.id} to complete "
            f"after {max_retries} retries (~{max_retries * poll_interval:.0f}s)"
        )

    async def evaluate(
        self,
        *,
        agent: str | Agent | Callable[[list[ChatMessage]], AgentReturn | Awaitable[AgentReturn]],
        dataset: str | Dataset,
        project: Optional[str | Project] | Omit = omit,
        name: Optional[str] | Omit = omit,
        tags: Optional[SequenceNotStr[str]] | Omit = omit,
    ) -> Evaluation:
        """Asynchronously run an evaluation for a given agent over a dataset.

        Handles both remote agents (referenced by ID or ``Agent``) and local
        Python callables (sync or async) that take a list of ``ChatMessage``
        and return an ``AgentOutput``-compatible value.

        Parameters
        ----------
        agent :
            Either a remote agent identifier (``str`` or ``Agent``) or a callable
            with signature ``(messages: list[ChatMessage]) -> AgentReturn``.
        dataset :
            Dataset identifier or ``Dataset`` instance containing the test cases
            to evaluate the agent on.
        project :
            Project identifier or ``Project`` instance.  Required when ``agent``
            is a remote agent (string or ``Agent``).
        name :
            Optional name to assign to the created evaluation.
        tags :
            Optional list of tags to filter the dataset's test cases.

        Returns
        -------
        Evaluation
            The created evaluation.

        Raises
        ------
        ValueError
            If ``project`` is not provided when running a remote evaluation.
        TypeError
            If the local agent callable returns an unsupported value, or if test
            cases do not include full ``TestCase`` objects during local evaluation.
        """
        dataset_id = dataset if isinstance(dataset, str) else dataset.id

        if isinstance(agent, (str, Agent)):
            return await self._evaluate_remote(
                agent=agent, dataset_id=dataset_id, project=project, name=name, tags=tags
            )

        return await self._evaluate_local(agent=agent, dataset_id=dataset_id, name=name, tags=tags)

    async def print_metrics(self, entity: PrintMetricsEntity) -> None:
        """Print metrics for an evaluation or scan result to the console (async).

        For an evaluation, displays a table of metric names, success rates, and
        pass/fail/error/skipped counts.  For a scan result, displays probe
        categories, names, severity, and issue/attack counts.
        """
        if isinstance(entity, Evaluation):
            print_evaluation_metrics_table(entity)
        else:
            await self._print_scan_metrics(entity)

    # -- Private helpers -----------------------------------------------------

    async def _evaluate_remote(
        self,
        *,
        agent: str | Agent,
        dataset_id: str,
        project: Optional[str | Project] | Omit,
        name: Optional[str] | Omit,
        tags: Optional[SequenceNotStr[str]] | Omit,
    ) -> Evaluation:
        if project is omit or project is None:
            raise ValueError("Project is required when running a remote evaluation")

        project_id = project if isinstance(project, str) else cast(Project, project).id
        agent_id = agent if isinstance(agent, str) else agent.id
        name_arg: str | Omit = omit if name is None else name

        return await self._client.evaluations.create(
            project_id=project_id,
            agent_id=agent_id,
            name=name_arg,
            dataset_id=dataset_id,
            tags=tags,
        )

    async def _evaluate_local(
        self,
        *,
        agent: Callable[[list[ChatMessage]], AgentReturn | Awaitable[AgentReturn]],
        dataset_id: str,
        name: Optional[str] | Omit,
        tags: Optional[SequenceNotStr[str]] | Omit,
    ) -> Evaluation:
        evaluation = await self._client.evaluations.create_local(
            agent_info={
                "name": agent.__name__,
                "description": agent.__doc__ or "",
            },
            name=name,
            dataset_id=dataset_id,
            tags=tags,
        )

        entries = await self._client.evaluations.results.list(
            evaluation_id=evaluation.id,
            include=["test_case"],
        )

        async def _process_entry(entry: TestCaseEvaluation) -> None:
            test_case = entry.test_case
            if not isinstance(test_case, TestCase):
                raise TypeError("Expected `test_case` to be a full TestCase for local evaluation")

            output = agent(test_case.messages)
            if inspect.isawaitable(output):
                output = await output

            agent_output_model = normalize_agent_output(output)
            agent_output_param = cast(AgentOutputParam, agent_output_model.to_dict())

            await self._client.evaluations.results.submit_local_output(
                result_id=entry.id,
                evaluation_id=evaluation.id,
                agent_output=agent_output_param,
            )

        await asyncio.gather(*(_process_entry(entry) for entry in entries))

        return evaluation

    async def _print_scan_metrics(self, entity: object) -> None:
        from ..types.scan import Scan as _Scan

        scan = cast(_Scan, entity)
        category_map = {cat.id: cat.title for cat in await self._client.scans.list_categories()}
        probe_results = await self._client.scans.list_probes(scan_id=scan.id)
        attempts_list = await asyncio.gather(
            *(self._client.scans.probes.list_attempts(probe_id=probe.id) for probe in probe_results)
        )
        attempts_by_probe_id = {
            probe.id: attempts for probe, attempts in zip(probe_results, attempts_list, strict=True)
        }
        probe_data = build_scan_probe_data(category_map, probe_results, attempts_by_probe_id)
        print_scan_metrics_table(probe_data, scan.id)



from __future__ import annotations

from typing import List, Tuple, Literal, Optional, overload

import httpx

from ..types import (
    Audit,
    AuditDisplay,
    AuditFiltersParam,
    AuditOrderByParam,
    AuditSearchParams,
    APIPaginatedResponse,
)
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..types.audit import AuditListEntityParams
from .._base_client import make_request_options
from ..types.common import APIPaginatedMetadata

__all__ = ["AuditLogsResource", "AsyncAuditLogsResource"]


class AuditLogsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AuditLogsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Giskard-AI/giskard-hub-python#accessing-raw-response-data-eg-headers
        """
        return AuditLogsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AuditLogsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Giskard-AI/giskard-hub-python#with_streaming_response
        """
        return AuditLogsResourceWithStreamingResponse(self)

    @overload
    def search(
        self,
        *,
        query: Optional[str] | Omit = omit,
        order_by: List[AuditOrderByParam] | Omit = omit,
        filters: AuditFiltersParam | Omit = omit,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
        include_metadata: bool = False,
    ) -> List[Audit]: ...

    @overload
    def search(
        self,
        *,
        query: Optional[str] | Omit = omit,
        order_by: List[AuditOrderByParam] | Omit = omit,
        filters: AuditFiltersParam | Omit = omit,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
        include_metadata: Literal[True],
    ) -> Tuple[List[Audit], APIPaginatedMetadata]: ...

    def search(
        self,
        *,
        query: Optional[str] | Omit = omit,
        order_by: List[AuditOrderByParam] | Omit = omit,
        filters: AuditFiltersParam | Omit = omit,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
        include_metadata: bool = False,
    ) -> List[Audit] | Tuple[List[Audit], APIPaginatedMetadata]:
        """Search audit logs using filters, sorting, and pagination.

        Parameters
        ----------
        query : str | None | Omit
            Free-text search query for audit logs.
        order_by : list[AuditOrderByParam] | Omit
            Sorting criteria for audit logs.
        filters : AuditFiltersParam | Omit
            Filter criteria for audit logs.
        limit : int | Omit
            Maximum number of results to return.
        offset : int | Omit
            Number of results to skip for pagination.
        include_metadata : bool
            If ``True``, return a tuple of ``(data, metadata)`` with pagination info.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        list[Audit] | tuple[list[Audit], APIPaginatedMetadata]
            Audit logs, optionally with pagination metadata.
        """
        response = self._post(
            "/v2/audit/search",
            body=maybe_transform(
                {
                    "filters": filters,
                    "order_by": order_by,
                    "search": query,
                },
                AuditSearchParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "offset": offset,
                    },
                    AuditSearchParams,
                ),
            ),
            cast_to=APIPaginatedResponse[Audit, None],
        )

        return self._unwrap_paginated(response, include_metadata)

    @overload
    def list_entities(
        self,
        entity_id: str,
        entity_type: str,
        *,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
        include_metadata: bool = False,
    ) -> List[AuditDisplay]: ...

    @overload
    def list_entities(
        self,
        entity_id: str,
        entity_type: str,
        *,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
        include_metadata: Literal[True],
    ) -> Tuple[List[AuditDisplay], APIPaginatedMetadata]: ...

    def list_entities(
        self,
        entity_id: str,
        entity_type: str,
        *,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
        include_metadata: bool = False,
    ) -> List[AuditDisplay] | Tuple[List[AuditDisplay], APIPaginatedMetadata]:
        """List audit display logs for a specific entity.

        Parameters
        ----------
        entity_id : str
            UUID of the entity to list audit logs for.
        entity_type : str
            Type of the entity (e.g. ``"project"``, ``"agent"``).
        limit : int | Omit
            Maximum number of results to return.
        offset : int | Omit
            Number of results to skip for pagination.
        include_metadata : bool
            If ``True``, return a tuple of ``(data, metadata)`` with pagination info.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        list[AuditDisplay] | tuple[list[AuditDisplay], APIPaginatedMetadata]
            Audit display logs, optionally with pagination metadata.

        Raises
        ------
        ValueError
            If ``entity_type`` or ``entity_id`` is empty.
        """
        if not entity_type:
            raise ValueError(f"Expected a non-empty value for `entity_type` but received {entity_type!r}")
        if not entity_id:
            raise ValueError(f"Expected a non-empty value for `entity_id` but received {entity_id!r}")
        response = self._get(
            f"/v2/audit/{entity_type}/{entity_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "offset": offset,
                    },
                    AuditListEntityParams,
                ),
            ),
            cast_to=APIPaginatedResponse[AuditDisplay, None],
        )

        return self._unwrap_paginated(response, include_metadata)


class AsyncAuditLogsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAuditLogsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Giskard-AI/giskard-hub-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAuditLogsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAuditLogsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Giskard-AI/giskard-hub-python#with_streaming_response
        """
        return AsyncAuditLogsResourceWithStreamingResponse(self)

    @overload
    async def search(
        self,
        *,
        query: Optional[str] | Omit = omit,
        order_by: Optional[List[AuditOrderByParam]] | Omit = omit,
        filters: Optional[AuditFiltersParam] | Omit = omit,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
        include_metadata: bool = False,
    ) -> List[Audit]: ...

    @overload
    async def search(
        self,
        *,
        query: Optional[str] | Omit = omit,
        order_by: Optional[List[AuditOrderByParam]] | Omit = omit,
        filters: Optional[AuditFiltersParam] | Omit = omit,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
        include_metadata: Literal[True],
    ) -> Tuple[List[Audit], APIPaginatedMetadata]: ...

    async def search(
        self,
        *,
        query: Optional[str] | Omit = omit,
        order_by: Optional[List[AuditOrderByParam]] | Omit = omit,
        filters: Optional[AuditFiltersParam] | Omit = omit,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
        include_metadata: bool = False,
    ) -> List[Audit] | Tuple[List[Audit], APIPaginatedMetadata]:
        """Search audit logs using filters, sorting, and pagination.

        Parameters
        ----------
        query : str | None | Omit
            Free-text search query for audit logs.
        order_by : list[AuditOrderByParam] | Omit
            Sorting criteria for audit logs.
        filters : AuditFiltersParam | Omit
            Filter criteria for audit logs.
        limit : int | Omit
            Maximum number of results to return.
        offset : int | Omit
            Number of results to skip for pagination.
        include_metadata : bool
            If ``True``, return a tuple of ``(data, metadata)`` with pagination info.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        list[Audit] | tuple[list[Audit], APIPaginatedMetadata]
            Audit logs, optionally with pagination metadata.
        """
        response = await self._post(
            "/v2/audit/search",
            body=await async_maybe_transform(
                {
                    "filters": filters,
                    "order_by": order_by,
                    "search": query,
                },
                AuditSearchParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "limit": limit,
                        "offset": offset,
                    },
                    AuditSearchParams,
                ),
            ),
            cast_to=APIPaginatedResponse[Audit, None],
        )

        return self._unwrap_paginated(response, include_metadata)

    @overload
    async def list_entities(
        self,
        entity_id: str,
        entity_type: str,
        *,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
        include_metadata: bool = False,
    ) -> List[AuditDisplay]: ...

    @overload
    async def list_entities(
        self,
        entity_id: str,
        entity_type: str,
        *,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
        include_metadata: Literal[True],
    ) -> Tuple[List[AuditDisplay], APIPaginatedMetadata]: ...

    async def list_entities(
        self,
        entity_id: str,
        entity_type: str,
        *,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
        include_metadata: bool = False,
    ) -> List[AuditDisplay] | Tuple[List[AuditDisplay], APIPaginatedMetadata]:
        """List audit display logs for a specific entity.

        Parameters
        ----------
        entity_id : str
            UUID of the entity to list audit logs for.
        entity_type : str
            Type of the entity (e.g. ``"project"``, ``"agent"``).
        limit : int | Omit
            Maximum number of results to return.
        offset : int | Omit
            Number of results to skip for pagination.
        include_metadata : bool
            If ``True``, return a tuple of ``(data, metadata)`` with pagination info.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        list[AuditDisplay] | tuple[list[AuditDisplay], APIPaginatedMetadata]
            Audit display logs, optionally with pagination metadata.

        Raises
        ------
        ValueError
            If ``entity_type`` or ``entity_id`` is empty.
        """
        if not entity_type:
            raise ValueError(f"Expected a non-empty value for `entity_type` but received {entity_type!r}")
        if not entity_id:
            raise ValueError(f"Expected a non-empty value for `entity_id` but received {entity_id!r}")
        response = await self._get(
            f"/v2/audit/{entity_type}/{entity_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "limit": limit,
                        "offset": offset,
                    },
                    AuditListEntityParams,
                ),
            ),
            cast_to=APIPaginatedResponse[AuditDisplay, None],
        )

        return self._unwrap_paginated(response, include_metadata)


class AuditLogsResourceWithRawResponse:
    def __init__(self, audit: AuditLogsResource) -> None:
        self._audit = audit

        self.search = to_raw_response_wrapper(
            audit.search,
        )
        self.list_entities = to_raw_response_wrapper(
            audit.list_entities,
        )


class AsyncAuditLogsResourceWithRawResponse:
    def __init__(self, audit: AsyncAuditLogsResource) -> None:
        self._audit = audit

        self.search = async_to_raw_response_wrapper(
            audit.search,
        )
        self.list_entities = async_to_raw_response_wrapper(
            audit.list_entities,
        )


class AuditLogsResourceWithStreamingResponse:
    def __init__(self, audit: AuditLogsResource) -> None:
        self._audit = audit

        self.search = to_streamed_response_wrapper(
            audit.search,
        )
        self.list_entities = to_streamed_response_wrapper(
            audit.list_entities,
        )


class AsyncAuditLogsResourceWithStreamingResponse:
    def __init__(self, audit: AsyncAuditLogsResource) -> None:
        self._audit = audit

        self.search = async_to_streamed_response_wrapper(
            audit.search,
        )
        self.list_entities = async_to_streamed_response_wrapper(
            audit.list_entities,
        )



"""Chat message and header types."""

from typing import Dict, Optional, TypedDict
from typing_extensions import Required

from .._models import BaseModel

__all__ = [
    "ChatMessage",
    "ChatMessageParam",
    "ChatMessageWithMetadata",
    "ChatMessageWithMetadataParam",
    "Header",
    "HeaderParam",
]


class ChatMessage(BaseModel):
    content: str
    role: str


class ChatMessageParam(TypedDict, total=False):
    content: Required[str]
    role: Required[str]


class ChatMessageWithMetadata(BaseModel):
    content: str
    role: str
    metadata: Optional[Dict[str, object]] = None


class ChatMessageWithMetadataParam(TypedDict, total=False):
    content: Required[str]
    role: Required[str]
    metadata: Optional[Dict[str, object]]


class Header(BaseModel):
    name: str
    value: str


class HeaderParam(TypedDict, total=False):
    name: Required[str]
    value: Required[str]
